"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var StringUtils_1 = require("./StringUtils");
var texton_1 = require("./texton");
var RangeUtils_1 = require("./RangeUtils");
var XSelection = /** @class */ (function () {
    function XSelection(range, xDocument) {
        this.range = range;
        this.xdoc = xDocument;
    }
    /**
     * create {@link XSelection} from native {@link Selection}
     * @param selection the native `Selection` from which to create the `XSelection` instance.
     * @param xDoc the context of the to be created `XSelection` instance.
     */
    XSelection.fromSelection = function (selection, xDoc) {
        var range = XSelection.rangeFromSelection(selection, xDoc);
        return XSelection.fromRange(range, false, xDoc);
    };
    /**
     * create {@link XSelection} from {@link ITextRange} pojo;
     * {@link ITextRange} specifies the start point and end point
     * <pre>
     * export interface ITextRange {
     *   from: { text: string, nth: number }, // means starting from the `nth` `text`
     *   to: { text: string, nth: number }, // means ending at the `nth` `text`
     * }
     * </pre>
     * @param textRange the serialized pojo from which to create the `XSelection` instance.
     * @param optSelect true to create explicit selection(be selected visibly from browser), false otherwise
     * @param xDoc the context of the to be created `XSelection` instance.
     */
    XSelection.fromTextRange = function (textRange, optSelect, xDoc) {
        if (optSelect === void 0) { optSelect = false; }
        xDoc.refresh();
        var range = XSelection.rangeFromTextRange(textRange, xDoc);
        return XSelection.fromRange(range, optSelect, xDoc);
    };
    /**
     * create {@link XSelection} from {@link ITextRange} pojo
     * @param index the (nth) text to be selected.
     * @param optSelect true to create explicit selection(be selected visibly from browser), false otherwise
     * @param xDoc the context of the to be created `XSelection` instance.
     */
    XSelection.fromText = function (index, optSelect, xDoc) {
        if (optSelect === void 0) { optSelect = false; }
        xDoc.refresh();
        var range = XSelection.rangeFromText(index, xDoc);
        return XSelection.fromRange(range, optSelect, xDoc);
    };
    /**
     * create {@link XSelection} from native {@link Range}
     * @param range the text content to be selected.
     * @param optSelect true to create selection explicitly(be selected visibly from browser), false otherwise
     * @param xDoc the context of the to be created `XSelection` instance.
     */
    XSelection.fromRange = function (range, optSelect, xDoc) {
        if (optSelect === void 0) { optSelect = false; }
        if (range) {
            range = RangeUtils_1.default.trim(range);
            if (optSelect) {
                var selection = xDoc.win.getSelection();
                if (selection) {
                    selection.removeAllRanges();
                    selection.addRange(range);
                }
            }
            return new XSelection(range, xDoc);
        }
        return null;
    };
    XSelection.rangeFromSelection = function (selection, xDoc) {
        if (selection.rangeCount > 0 && !selection.isCollapsed) {
            return selection.getRangeAt(0);
        }
        return null;
    };
    XSelection.rangeFromTextRange = function (textRange, xDoc) {
        var head = XSelection.rangeFromText(textRange.start, xDoc);
        var tail = XSelection.rangeFromText(textRange.end, xDoc);
        if (head && tail) {
            var range = xDoc.doc.createRange();
            range.setStart(head.endContainer, head.endOffset);
            range.setEnd(tail.endContainer, tail.endOffset);
            return range;
        }
        return null;
    };
    /**
     *  return an empty range object that has both of its boundary points positioned at
     *  the beginning of the document when no parameters are given, else return an range
     *  object that contains the nth occurrence of the specified text
     *
     * @param {ITextIndex} index
     * @param {XDocument} xDoc
     * @return {Range} the range created that contains the specified text
     * @private
     */
    XSelection.rangeFromText = function (index, xDoc) {
        var text = index.text, nth = index.nth;
        var nodes = xDoc.nodes;
        var cText = StringUtils_1.StringUtils.compact(text);
        var i = StringUtils_1.StringUtils.indexOf(xDoc.text, cText, nth || 1);
        if (i > -1) {
            var sPos_1 = i + nodes[0].startPosition; // include the first character
            var ePos_1 = sPos_1 + cText.length; // exclude the last character
            var sIndex = nodes.findIndex(function (node) { return node.endPosition >= sPos_1; });
            var eIndex = nodes.findIndex(function (node) { return node.startPosition >= ePos_1; }) - 1;
            var sContainer = nodes[sIndex];
            var eContainer = eIndex >= 0 ? nodes[eIndex] : nodes[nodes.length - 1];
            if (sContainer) {
                var sOffset = StringUtils_1.StringUtils.indexOfNthNotEmptyChar(sContainer.data, sPos_1 - sContainer.startPosition);
                var eOffset = StringUtils_1.StringUtils.indexOfNthNotEmptyChar(eContainer.data, ePos_1 - eContainer.startPosition - 1);
                var range = xDoc.doc.createRange();
                range.setStart(sContainer, sOffset);
                range.setEnd(eContainer, eOffset + 1);
                return range;
            }
        }
        return null;
    };
    XSelection.prototype.trim = function () {
        var selection = this.xdoc.win.getSelection();
        if (selection) {
            selection.removeAllRanges();
            selection.addRange(RangeUtils_1.default.trim(this.range));
        }
    };
    /**
     * the text nodes contained in the selection.
     * <em>
     * text node will be splatted into 2 parts if the selection is start from/end in a internal
     * position of a text node
     * </em>
     */
    XSelection.prototype.getTextNodes = function () {
        if (this.texts) {
            return this.texts;
        }
        this.texts = RangeUtils_1.default.extractTextNodes(this.range);
        var selection = this.xdoc.win.getSelection();
        if (selection && selection.rangeCount > 0) {
            selection.collapseToStart();
            selection.empty && selection.empty();
            selection.removeAllRanges && selection.removeAllRanges();
        }
        return this.texts;
    };
    /**
     * get the exact "occurrence" of this selection.
     * <em>
     *   the selected text content may appear >=1 times in this page, and the selection is it's
     *   {@code occurrence.nth} time. and the start char of this selection is the
     *   {@code occurrence.position}th char of the whole text content in this page.
     * </em>
     */
    XSelection.prototype.serializeToIndex = function () {
        var cText = StringUtils_1.StringUtils.compact(this.range.toString());
        if (cText.length < 1) {
            return texton_1.ITextIndex.None;
        }
        var content = texton_1.NodeUtils.getContentTo(this.range.endContainer, this.range.endOffset, this.xdoc);
        var texts = StringUtils_1.StringUtils.find(content, cText);
        if (texts && texts.length > 0) {
            return texts[texts.length - 1];
        }
        return texton_1.ITextIndex.None;
    };
    /**
     * get the text range of the selection in this page.
     * <pre>
     * export interface ITextRange {
     *   from: { text: string, nth: number }, // means starting from the `nth` `text`
     *   to: { text: string, nth: number }, // means ending at the `nth` `text`
     * }
     * </pre>
     * that is, the text content of this selection starts with {@code textRange.from.text} and
     * ends with {@code textRange.to.text}. and {@code textRange.from.text} is its {@code textRange.from.nth}
     * occurrence, {@code textRange.to.text} is its {@code textRange.to.nth} occurrence.
     * e.g.
     * for text
     * <span>hello world, hello miller, hello world, hello miller, hello miller</span>
     * <pre>
     * {
     *   from:{text: 'hello', nth: 2}
     *   to:{text: 'miller', nth: 3}
     * }
     * means, the selection starts from the 2nd `hello` and ends with 3rd `miller`. that is:
     * <span>hello world, <selected>hello miller, hello world, hello miller, hello miller<selected>, hello world<span>
     * </pre>
     *
     * @param len
     */
    XSelection.prototype.serializeToRange = function (len) {
        if (len === void 0) { len = 5; }
        var cText = StringUtils_1.StringUtils.compact(this.range.toString());
        if (cText.length < 1) {
            return texton_1.ITextRange.None;
        }
        var eContent = texton_1.NodeUtils.getContentTo(this.range.endContainer, this.range.endOffset, this.xdoc);
        var eText = eContent.length > len ? eContent.substr(eContent.length - len, len) : eContent;
        var eTexts = StringUtils_1.StringUtils.find(eContent, eText);
        var sContent = texton_1.NodeUtils.getContentTo(this.range.startContainer, this.range.startOffset, this.xdoc);
        var sText = sContent.length > len ? sContent.substr(sContent.length - len, len) : sContent;
        var sTexts = StringUtils_1.StringUtils.find(sContent, sText);
        if (sTexts && sTexts.length > 0 && eTexts && eTexts.length > 0) {
            return {
                start: sTexts[sTexts.length - 1],
                end: eTexts[eTexts.length - 1]
            };
        }
        return texton_1.ITextRange.None;
    };
    /**
     * get the selected text content
     * @param compact true to remove the white spaces. false otherwise
     */
    XSelection.prototype.getContent = function (compact) {
        if (compact === void 0) { compact = false; }
        if (compact) {
            return StringUtils_1.StringUtils.compact(this.range.toString());
        }
        return this.range.toString();
    };
    /**
     * get the native selection
     */
    XSelection.prototype.toSelection = function () {
        var selection = this.xdoc.win.getSelection();
        if (selection.rangeCount < 1) {
            selection.addRange(this.range);
        }
        return selection;
    };
    /**
     * clear the selection
     */
    XSelection.prototype.empty = function () {
        if (!this.texts || this.texts.length < 1) {
            return;
        }
        var first = this.range.startContainer;
        var last = this.range.endContainer;
        if (first && first.parentNode) {
            first.parentNode.normalize();
        }
        if (last && last.parentNode) {
            last.parentNode.normalize();
        }
        this.range.collapse();
        this.texts = [];
        var selection = this.xdoc.win.getSelection();
        selection.collapseToStart();
        selection.empty && selection.empty();
        selection.removeAllRanges && selection.removeAllRanges();
    };
    /**
     * check if the selection is cleared or not.
     */
    XSelection.prototype.isEmpty = function () {
        var selection = this.xdoc.win.getSelection();
        return !selection || selection.rangeCount < 1 || selection.isCollapsed || this.range.collapsed;
    };
    return XSelection;
}());
exports.XSelection = XSelection;
//# sourceMappingURL=XSelection.js.map